@@ -119,6 +119,35 @@ mk_hash_index(struct tuple4 addr)
   return hash % tcp_stream_table_size;
 }
 
+static int get_ts(struct tcphdr * this_tcphdr, unsigned int * ts)
+{
+  int len = 4 * this_tcphdr->th_off;
+  unsigned int tmp_ts;
+  unsigned char * options = (char*)(this_tcphdr + 1);
+  int ind = 0, ret = 0;
+  while (ind <=  len - sizeof (struct tcphdr) - 10 )
+  	switch (options[ind]) {
+		case 0: /* TCPOPT_EOL */
+			return ret;
+		case 1: /* TCPOPT_NOP */
+			ind++;
+			continue;	
+  		case 8: /* TCPOPT_TIMESTAMP */
+	  		memcpy((char*)&tmp_ts, options + ind + 2, 4);
+  			*ts=ntohl(tmp_ts);
+			ret = 1;
+			/* no break, intentionally */
+		default:	
+			if (options[ind+1] < 2 ) /* "silly option" */
+				return ret;
+			ind += options[ind+1];
+	}			
+			
+  return ret;
+}  		
+    
+
+
 static void
 add_new_tcp(struct tcphdr * this_tcphdr, struct ip * this_iphdr)
 {
@@ -158,6 +187,7 @@ add_new_tcp(struct tcphdr * this_tcphdr, struct ip * this_iphdr)
   a_tcp->client.seq = ntohl(this_tcphdr->th_seq) + 1;
   a_tcp->client.first_data_seq = a_tcp->client.seq;
   a_tcp->client.window = ntohs(this_tcphdr->th_win);
+  a_tcp->client.ts_on = get_ts(this_tcphdr, &a_tcp->client.curr_ts);
   a_tcp->server.state = TCP_CLOSE;
   a_tcp->next_node = tolink;
   a_tcp->prev_node = 0;
@@ -176,10 +206,10 @@ add_new_tcp(struct tcphdr * this_tcphdr, struct ip * this_iphdr)
 static void
 add2buf(struct half_stream * rcv, char *data, int datalen)
 {
+  int toalloc;
+  
   if (datalen + rcv->count - rcv->offset > rcv->bufsize) {
     if (!rcv->data) {
-      int toalloc;
-      
       if (datalen < 2048)
 	toalloc = 4096;
       else
@@ -188,8 +218,12 @@ add2buf(struct half_stream * rcv, char *data, int datalen)
       rcv->bufsize = toalloc;
     }
     else {
-      rcv->data = realloc(rcv->data, 2 * rcv->bufsize);
-      rcv->bufsize *= 2;
+      if (datalen < rcv->bufsize)
+      	toalloc = 2 * rcv->bufsize;
+      else	
+      	toalloc = rcv->bufsize + 2*datalen;
+      rcv->data = realloc(rcv->data, toalloc);
+      rcv->bufsize = toalloc;
     }
     if (!rcv->data)
       nids_params.no_mem("add2buf");
@@ -309,6 +343,7 @@ add_from_skb(struct tcp_stream * a_tcp, struct half_stream * rcv,
       }
       else {
 	rcv->count += to_copy;
+	rcv->offset = rcv->count; /* clear the buffer */
       }
     }
     rcv->urgdata = data[rcv->urg_ptr - this_seq];
@@ -324,6 +359,7 @@ add_from_skb(struct tcp_stream * a_tcp, struct half_stream * rcv,
       }
       else {
 	rcv->count += to_copy2;
+	rcv->offset = rcv->count; /* clear the buffer */
       }
     }
   }
@@ -335,6 +371,7 @@ add_from_skb(struct tcp_stream * a_tcp, struct half_stream * rcv,
       }
       else {
 	rcv->count += datalen - lost;
+	rcv->offset = rcv->count; /* clear the buffer */
       }
     }
   }
@@ -358,6 +395,7 @@ tcp_queue(struct tcp_stream * a_tcp, struct tcphdr * this_tcphdr,
   if (!after(this_seq, EXP_SEQ)) {
     if (after(this_seq + datalen + (this_tcphdr->th_flags & TH_FIN), EXP_SEQ)) {
       /* the packet straddles our window end */
+      get_ts(this_tcphdr, &snd->curr_ts);
       add_from_skb(a_tcp, rcv, snd, data, datalen, this_seq,
 		   (this_tcphdr->th_flags & TH_FIN),
 		   (this_tcphdr->th_flags & TH_URG),
@@ -420,6 +458,8 @@ tcp_queue(struct tcp_stream * a_tcp, struct tcphdr * this_tcphdr,
     if (!p) {
       pakiet->prev = 0;
       pakiet->next = rcv->list;
+      if (rcv->list)
+         rcv->list->prev = pakiet;
       rcv->list = pakiet;
       if (!rcv->listtail)
 	rcv->listtail = pakiet;
@@ -509,6 +549,24 @@ find_stream(struct tcphdr * this_tcphdr, struct ip * this_iphdr,
     return 0;
 }
 
+void clear_stream_buffers()
+{
+  int i;
+  struct lurker_node *j;
+  struct tcp_stream *a_tcp;
+  
+  for (i = 0; i < tcp_stream_table_size; i++) {
+    for (a_tcp = tcp_stream_table[i];
+         a_tcp;
+         a_tcp = a_tcp->next_node) {
+      for (j = a_tcp->listeners; j; j = j->next) {
+          a_tcp->nids_state = NIDS_EXITING;
+          (j->item)(a_tcp, &j->data);
+      }
+    }
+  }
+}
+        
 void
 process_tcp(u_char * data, int skblen)
 {
@@ -516,6 +574,7 @@ process_tcp(u_char * data, int skblen)
   struct tcphdr *this_tcphdr = (struct tcphdr *)(data + 4 * this_iphdr->ip_hl);
   int datalen, iplen;
   int from_client = 1;
+  unsigned int tmp_ts;
   struct tcp_stream *a_tcp;
   struct half_stream *snd, *rcv;
 
@@ -577,6 +636,11 @@ process_tcp(u_char * data, int skblen)
     a_tcp->server.first_data_seq = a_tcp->server.seq;
     a_tcp->server.ack_seq = ntohl(this_tcphdr->th_ack);
     a_tcp->server.window = ntohs(this_tcphdr->th_win);
+    if (a_tcp->client.ts_on) {
+    	a_tcp->server.ts_on = get_ts(this_tcphdr, &a_tcp->server.curr_ts);
+	if (!a_tcp->server.ts_on)
+		a_tcp->client.ts_on = 0;
+    } else a_tcp->server.ts_on = 0;	
     return;
   }
   if (!before(ntohl(this_tcphdr->th_seq), rcv->ack_seq + rcv->window) ||
@@ -594,6 +658,12 @@ process_tcp(u_char * data, int skblen)
     free_tcp(a_tcp);
     return;
   }
+
+  /* PAWS check */
+  if (rcv->ts_on && get_ts(this_tcphdr, &tmp_ts) && 
+  	before(tmp_ts, snd->curr_ts))
+  return; 	
+  
   if ((this_tcphdr->th_flags & TH_ACK)) {
     if (from_client && a_tcp->client.state == TCP_SYN_SENT &&
 	a_tcp->server.state == TCP_SYN_RECV) {
